/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#if !defined(HARK_BASE_TRANSFERFUNCTION_H_INCLUDED)
#define HARK_BASE_TRANSFERFUNCTION_H_INCLUDED 1

#include "Map.h"
#include "Matrix.h"
#include "Vector.h"

using namespace std;

namespace HARK
{
class TransferFunction;

enum TF_Coordinate {
	Polar,		// 極座標系
	Cartesian	// 直交座標系
};

enum TF_NeighborAlgorithm {
	Undefined,		// 未設定。デフォルト値
	NearestNeighbor	// Nearest Neighbor 法。　閾値より近い複数個の Position が隣接していると判定する
};

class Position : public Object {
public:
	int						id;				// 位置の ID
	TF_Coordinate			sys;			// メンバ coord の座標系
	float					coord[3];		// 座標
	String					path;			// この位置に対応するファイルのパス。TSP録音ファイルなど
	Matrix<complex<float>>	data;			// この位置に対応する行列データ。伝達関数など
	int						channelsUse;	// この位置に対応するチャンネルセット情報（numChannels,channels）、0:未使用／1:使用
	Vector<int>				channels;		// この位置に対応するチャンネルの数

	Position(){
	}
	Position(const Position &data){
		*this = data;
	}
	virtual ObjectRef clone();
	virtual void printOn(std::ostream &out) const;
	void readFrom(std::istream &in){
	}
	Position& operator = (const Position &src){
		id = src.id;
		sys = src.sys;
		coord[0] = src.coord[0];
		coord[1] = src.coord[1];
		coord[2] = src.coord[2];
		path = src.path;
		data.resize(src.data.nrows(), src.data.ncols());
		for (int i=0; i<src.data.nrows(); i++){
			for (int j=0; j<src.data.ncols(); j++){
				data(i, j) = src.data(i, j);
			}
		}
		channelsUse = src.channelsUse;
		channels = src.channels;
		return *this;
	}
};
std::istream &operator >> (std::istream &in, Position &data);

class Neighbor : public Object {
public:
	Vector<int>					ids;		// 隣接関係を表したいPositionのidのリスト
	Vector<Vector<int>>			neighbors;	// Positionごとの、隣接するPositionのidのリスト
	Vector<Vector<Position>>	poses;		// 隣接関係が表す Posisition のリスト
	TF_NeighborAlgorithm		alg;		// 隣接関係を求めるアルゴリズム

	Neighbor(){
		clear();
	}
	Neighbor(const Neighbor &data){
		*this = data;
	}
	virtual ObjectRef clone();
	virtual void printOn(std::ostream &out) const;
	void readFrom(std::istream &in){
	}
	void clear(){
		ids.clear();
		neighbors.clear();
		poses.clear();
		alg = NearestNeighbor;
	}
	Neighbor& operator = (const Neighbor &src){
		ids = src.ids;
		neighbors = src.neighbors;
		poses = src.poses;
		alg = src.alg;
		return *this;
	}
};
std::istream &operator >> (std::istream &in, Neighbor &data);

class Config : public Object {
public:
	String	comment;			// コメントに使用する任意の文字列
	int		synchronousAverage;	// TSPの同期加算回数
	String	TSPpath;			// TSPの元信号へのパス
	int		TSPoffset;			// TSPが開始するまでのオフセット [サンプル]
	int		TSPLength;			// 収録するTSP信号の長さ [サンプル]
	int		peakSearchFrom;		// インパルス応答のピーク検索範囲の開始位置
	int		peakSearchTo;		// インパルス応答のピーク検索範囲の終了位置
	int		nfft;				// FFT長
	int		samplingRate;		// 音響信号ファイルのサンプリング周波数
	int		signalMax;			// 音響信号ファイルの振幅の最大値

	Config(){
		clear();
	}
	Config(const Config &data){
		*this = data;
	}
	virtual ObjectRef clone();
	virtual void printOn(std::ostream &out) const;
	void readFrom(std::istream &in){
	}
	void clear(){
		comment = "";
		synchronousAverage = 0;
		TSPpath = "";
		TSPoffset = 0;
		TSPLength = 0;
		peakSearchFrom = 0;
		peakSearchTo = 0;
		nfft = 0;
		samplingRate = 0;
		signalMax = 0;
	}
};
std::istream &operator >> (std::istream &in, Config &data);

class TransferFunction : public Object {
private:
	String								m_WhatIsThis;
	Vector<Position>					m_Sources;
	Neighbor							m_Neighbors;
	Vector<Position>					m_Microphones;
	Config								m_Config;
	Map<int, Matrix<complex<float>>>	m_Localization;
	Map<int, Matrix<complex<float>>>	m_Separation;

public:
	TransferFunction(){
		m_WhatIsThis = "";
		m_Sources.clear();
		m_Neighbors.clear();
		m_Microphones.clear();
		m_Config.clear();
		m_Localization.clear();
		m_Separation.clear();
	}
	TransferFunction(const TransferFunction &data){
		*this = data;
	}

	virtual ObjectRef clone();
	virtual void printOn(std::ostream &out) const;
	void readFrom(std::istream &in){
	}
	void CopyMap(const Map<int, Matrix<complex<float>>> &src, Map<int, Matrix<complex<float>>> &dst) const {
		dst.clear();
		for (const auto &mp : src){
			Matrix<complex<float>>	mt;
			CopyMatrix(mp.second, mt);
			dst.insert(make_pair(mp.first, mt));
		}
	}

	void CopyMatrix(const Matrix<complex<float>> &src, Matrix<complex<float>> &dst) const {
		dst.resize(src.nrows(), src.ncols());
		for (int i=0; i<src.nrows(); i++){
			for (int j=0; j<src.ncols(); j++){
				dst(i, j) = src(i, j);
			}
		}
	}

	const String& GetWhatIsThis() const {
		return m_WhatIsThis;
	}
	void SetWhatIsThis(const String &whatIsThis){
		m_WhatIsThis = whatIsThis;
	}

	const Vector<Position>& GetSources() const {
		return m_Sources;
	}
	void SetSources(const Vector<Position> &sources){
		m_Sources = sources;
	}

	const Neighbor& GetNeighbors() const {
		return m_Neighbors;
	}
	void SetNeighbors(const Neighbor &neighbors){
		m_Neighbors = neighbors;
	}

	const Vector<Position>& GetMicrophones() const {
		return m_Microphones;
	}
	void SetMicrophones(const Vector<Position> &microphones){
		m_Microphones = microphones;
	}

	const Config& GetConfig() const {
		return m_Config;
	}
	void SetConfig(const Config &config){
		m_Config = config;
	}

	const Map<int, Matrix<complex<float>>>& GetLocalization() const {
		return m_Localization;
	}
	void SetLocalization(const Map<int, Matrix<complex<float>>> &localization){
		CopyMap(localization, m_Localization);
	}

	const Map<int, Matrix<complex<float>>>& GetSeparation() const {
		return m_Separation;
	}
	void SetSeparation(const Map<int, Matrix<complex<float>>> &separation){
		CopyMap(separation, m_Separation);
	}
};
std::istream &operator >> (std::istream &in, TransferFunction &data);

}//namespace end

#endif
//end of file
